// C Source File
// Created 07/08/2004; 15:19:23

#include <tigcclib.h>
#include "game.h"
#include "kbd.h"
#include "display.h"
#include "constants.h"
#include "tiles.h"

#define MAXSIZE 64

void menu();
void select();
void single();
void block();

void save();
void reload();
void size();
void help();

extern unsigned char * map;
Tiles * t;
extern unsigned char mapname[20];
extern unsigned char tilename[20];
extern unsigned char tileselname[20];
extern unsigned short dimX;
extern unsigned short dimY;
unsigned char menumode;
unsigned char entry;
unsigned short tile;
unsigned short mode;
unsigned short xmap;
unsigned short ymap;
unsigned short xcursorstart;
unsigned short ycursorstart;
unsigned short xcursor;
unsigned short ycursor;
unsigned short dimxblock;
unsigned short dimyblock;
unsigned short sizex;
unsigned short sizey;
unsigned char sizeentry;
unsigned char * blok;

void game()
{
	t = TilesLoad(tilename, tileselname);
	entry = 0;
	tile = 0;
	mode = 0;
	
	xmap = 0;
	ymap = 0;
	xcursor = 0;
	ycursor = 0;
	dimxblock = 0;
	dimyblock = 0;
	blok = NULL;
	menumode = 1;
	
	// main loop
  SetLoopFunc(menu);
  LoopInit();
  MainLoop();
	LoopEnd();
	if (blok)
		free(blok);
	TilesFree (t);
}

void menu()
{
	kbd_Repeat();
	
	if (R_ESC && map!=NULL)
	{
		menumode = 0;
		switch (mode)
		{
			case 0 :
				xcursorstart = xcursor;
				ycursorstart = ycursor;
				SetLoopFunc(select);
				break;
			case 1 :
				SetLoopFunc(single);
				break;
			case 2 :
				SetLoopFunc(block);
				break;
			default :;
		}
		return;
	}
	if (R_LEFT)
	{
		if (entry > 0)
			entry--;
		else
			entry =6;		
	}

	if (R_RIGHT)
	{
		if (entry < 6)
			entry++;
		else
			entry = 0;
	}

	switch (entry)
	{
		case 0:
			if(C)
			{
				if (UP)
					if (tile > 0)
						tile--;
				if (DOWN)
					if (tile < t->nbTiles - 1)
						tile++;
			}
			else
			{
				if (R_UP)
					if (tile > 0)
						tile--;
				if (R_DOWN)
					if (tile < t->nbTiles - 1)
						tile++;
			}			
			break;
		case 1:
			if (R_UP)
				if (mode > 0)
					mode--;
			if (R_DOWN)
				if (mode < 1 || (mode < 2 && blok))
					mode++;
			break;
		case 2:
			if (R_ENTER)
				save();
			break;
		case 3:
			if (R_ENTER)
				reload();
			break;
		case 4:
			if (R_ENTER || R_DOWN)
			{
				menumode = 2;
				sizeentry = 0;
				sizex = dimX;
				sizey = dimY;
				SetLoopFunc(size);
			}
			break;
		case 5:
			if (R_ENTER || R_DOWN)
			{
				menumode = 3;
				SetLoopFunc(help);
			}
			break;
		case 6:
			if (R_ENTER)
				LoopQuit();
			break;
		default:;
	}
}

unsigned char movestartcursor()
{
	unsigned char retval = 0;
	
	if(C)
	{	
		if (UP)
		{
			if(ycursorstart)
				ycursorstart--;		
			if((signed short)(ycursorstart - ymap) < 0)
				retval |= 0x01;
		}
		if (DOWN)
		{
			if(ycursorstart < dimY - 1)
				ycursorstart++;
			if(ycursorstart - ymap + 1 > LARGY)
				retval |= 0x02;
		}
		if (LEFT)
		{
			if(xcursorstart)
				xcursorstart--;
			if((signed short)(xcursorstart - xmap) < 0)
				retval |= 0x04;		
		}
		if (RIGHT)
		{
			if(xcursorstart < dimX - 1)
				xcursorstart++;
			if(xcursorstart - xmap + 1 > LARGX)
				retval |= 0x08;	
		}
	}
	else
	{
		if (R_UP)
		{
			if(ycursorstart)
				ycursorstart--;		
			if((signed short)(ycursorstart - ymap) < 0)
				retval |= 0x01;
		}
		if (R_DOWN)
		{
			if(ycursorstart < dimY - 1)
				ycursorstart++;
			if(ycursorstart - ymap + 1 > LARGY)
				retval |= 0x02;
		}
		if (R_LEFT)
		{
			if(xcursorstart)
				xcursorstart--;
			if((signed short)(xcursorstart - xmap) < 0)
				retval |= 0x04;		
		}
		if (R_RIGHT)
		{
			if(xcursorstart < dimX - 1)
				xcursorstart++;
			if(xcursorstart - xmap + 1 > LARGX)
				retval |= 0x08;	
		}
	}
	if (xcursorstart >= dimX)
		xcursorstart = dimX - 1;		
	if (ycursorstart >= dimY)
		ycursorstart = dimY - 1;		
			
	return retval;
}

unsigned char movecursor()
{
	unsigned char retval = 0;
	
	if(C)
	{	
		if (UP)
		{
			if(ycursor > ycursorstart)
				ycursor--;		
			if((signed short)(ycursor - ymap) < 0)
				retval |= 0x01;
		}
		if (DOWN)
		{
			if(ycursor < dimY - 1)
				ycursor++;
			if(ycursor - ymap + 1 > LARGY)
				retval |= 0x02;
		}
		if (LEFT)
		{
			if(xcursor > xcursorstart)
				xcursor--;
			if((signed short)(xcursor - xmap) < 0)
				retval |= 0x04;		
		}
		if (RIGHT)
		{
			if(xcursor < dimX - 1)
				xcursor++;
			if(xcursor - xmap + 1 > LARGX)
				retval |= 0x08;	
		}
	}
	else
	{
		if (R_UP)
		{
			if(ycursor > ycursorstart)
				ycursor--;		
			if((signed short)(ycursor - ymap) < 0)
				retval |= 0x01;
		}
		if (R_DOWN)
		{
			if(ycursor < dimY - 1)
				ycursor++;
			if(ycursor - ymap + 1 > LARGY)
				retval |= 0x02;
		}
		if (R_LEFT)
		{
			if(xcursor > xcursorstart)
				xcursor--;
			if((signed short)(xcursor - xmap) < 0)
				retval |= 0x04;		
		}
		if (R_RIGHT)
		{
			if(xcursor < dimX - 1)
				xcursor++;
			if(xcursor - xmap + 1 > LARGX)
				retval |= 0x08;	
		}
	}
	if (xcursor >= dimX)
		xcursor = dimX - 1;		
	if (ycursor >= dimY)
		ycursor = dimY - 1;		
			
	return retval;
}

void view()
{
	kbd_Repeat();
	if(!B)
	{
		switch (mode)
		{
			case 0 :
				SetLoopFunc(select);
				break;
			case 1 :
				SetLoopFunc(single);
				break;
			case 2 :
				SetLoopFunc(block);
				break;
			default :;
		}
		return;		
	}
	if(C)
	{
		if (UP)
			if(ymap)
				ymap--;
		if (DOWN)
			if(ymap + LARGY < dimY)
				ymap++;
		if (LEFT)
			if(xmap)
				xmap--;
		if (RIGHT)
			if(xmap + LARGX < dimX)
				xmap++;
	}
	else
	{
		if (R_UP)
			if(ymap)
				ymap--;
		if (R_DOWN)
			if(ymap + LARGY < dimY)
				ymap++;
		if (R_LEFT)
			if(xmap)
				xmap--;
		if (R_RIGHT)
			if(xmap + LARGX < dimX)
				xmap++;
	}
}

void select()
{
	unsigned char scroll = 0;
	register unsigned short i,j;
	kbd_Repeat();
	
	if (B)
	{
		SetLoopFunc(view);
		return;
	}
	if (A)
		scroll = movecursor();
	else
	{
		if ((xcursor == xcursorstart) && (ycursor == ycursorstart))
		{
			scroll = movestartcursor();
			xcursor = xcursorstart;		
			ycursor = ycursorstart;		
		}
	}
		
	if (scroll & 0x01)
		if(ymap)
			ymap--;
	if (scroll & 0x02)
		if(ymap + LARGY < dimY)
			ymap++;
	if (scroll & 0x04)
		if(xmap)
			xmap--;
	if (scroll & 0x08)
		if(xmap + LARGX < dimX)
			xmap++;
	
	if (R_ENTER)
	{
		if (blok)
			free(blok);
		dimxblock = xcursor - xcursorstart + 1;
		dimyblock = ycursor - ycursorstart + 1;
		
		blok = (unsigned char*)malloc(dimxblock * dimyblock * sizeof (unsigned char));
		for (i = 0; i<dimxblock; i++)
		{
			for (j = 0; j<dimyblock; j++)
			{
				blok[i + j * dimxblock] = map[(i + xcursorstart) + (j + ycursorstart) * dimX];
			}		
		}
		mode = 2;
		SetLoopFunc(block);
	}
	
	if (R_ESC)
	{
		menumode = 1;
		SetLoopFunc(menu);
	}
}

void single()
{
	unsigned char scroll = 0;

	kbd_Repeat();

	if (B)
	{
		SetLoopFunc(view);
		return;
	}

	scroll = movestartcursor();
	xcursor = xcursorstart;
	ycursor = ycursorstart;
		
	if (scroll & 0x01)
		if(ymap)
			ymap--;
	if (scroll & 0x02)
		if(ymap + LARGY < dimY)
			ymap++;
	if (scroll & 0x04)
		if(xmap)
			xmap--;
	if (scroll & 0x08)
		if(xmap + LARGX < dimX)
			xmap++;
	
	if(A)
	{
		map[xcursor + ycursor * dimX] = tile;
	}
	
	if (R_ESC)
	{
		menumode = 1;
		SetLoopFunc(menu);
	}
}

void block()
{
	register unsigned short i,j;
	unsigned char scroll = 0;

	kbd_Repeat();

	scroll = movestartcursor();
		
	if (scroll & 0x01)
		if(ymap)
			ymap--;
	if (scroll & 0x02)
		if(ymap + LARGY < dimY)
			ymap++;
	if (scroll & 0x04)
		if(xmap)
			xmap--;
	if (scroll & 0x08)
		if(xmap + LARGX < dimX)
			xmap++;	

	xcursor = xcursorstart + dimxblock - 1;
	ycursor = ycursorstart + dimyblock - 1;

	if (B)
	{
		SetLoopFunc(view);
		return;
	}
	
	if (R_A)
	{
		for(i=0; i<dimxblock; i++)
		{
			for(j=0; j<dimyblock; j++)
			{
				if ((i + xcursorstart < dimX) && (j + ycursorstart < dimY))
					map[(i + xcursorstart) + ((j + ycursorstart) * dimX)] = blok[i + j * dimxblock];
			}			
		}
	}
	
	if (R_ESC)
	{
		menumode = 1;
		SetLoopFunc(menu);
	}
}

void save()
{
	FILE * mapFile;
	register unsigned short i;
	if ((mapFile = fopen (mapname, "wb")) != NULL)
	{
		fwrite (&(dimX), sizeof(short), 1, mapFile); 
		fwrite (&(dimY), sizeof(short), 1, mapFile);
		
		for(i=0;i < (dimX * dimY);i++)
		{
			fwrite (&(map[i]), sizeof(char), 1, mapFile);
		}
		fputc (0, mapFile);
		fputs ("MAP", mapFile);
		fputc (0, mapFile);
		fputc (OTH_TAG, mapFile);
		fclose(mapFile);
	}
}

void reload()
{
	FILE * mapFile;
	register unsigned short i;
	if ((mapFile = fopen (mapname, "rb")) != NULL)
	{
		fread (&(dimX), sizeof(short), 1, mapFile); 
		fread (&(dimY), sizeof(short), 1, mapFile);
		map = (unsigned char *)malloc(sizeof(unsigned char) * dimX * dimY);
		
		for(i=0;i < (dimX * dimY);i++)
		{
			fread (&(map[i]), sizeof(char), 1, mapFile);
		}
		fclose (mapFile);
	}
}

void size()
{
	register unsigned short i, j;
	unsigned char * map2;
	INT_HANDLER old_int_1;

	
	kbd_Repeat();
	
	if (R_ESC || (R_UP && (sizeentry == 0)))
	{
		old_int_1 = GetIntVec (AUTO_INT_1); // auto int 1 off
		SetIntVec (AUTO_INT_1, DUMMY_HANDLER);

		if ((sizex != 0) && (sizey != 0) && ((sizex != dimX) || (sizey != dimY)))
		{
			if ((map2 = (unsigned char *)malloc(sizex * sizey * sizeof(unsigned char))))
			{
				for(i=0; i<sizex; i++)
				{
					for(j=0; j<sizey; j++)
					{
						if ((i<dimX) && (j<dimY))
							map2[i + j * sizex] = map[i + j * dimX];
						else
							map2[i + j * sizex] = tile;
					}
				}
				if(map)
						free (map);
					map = map2;
				dimX = sizex;
				dimY = sizey;
				
			}		
		}
		menumode = 1;
		SetLoopFunc(menu);
		SetIntVec (AUTO_INT_1, old_int_1); // auto int 1 on
		return;
	}
	
	if (R_UP)
		if (sizeentry > 0)
			sizeentry--;
	if (R_DOWN)
		if (sizeentry < 1)
			sizeentry++;
	switch (sizeentry)
	{
		case 0:
			if (C)
			{
				if (LEFT)
					if(sizex > 1)
						sizex --;
				if (RIGHT)
					if(sizex < MAXSIZE)
						sizex ++;
			}
			else
			{
				if (R_LEFT)
					if(sizex > 1)
						sizex --;
				if (R_RIGHT)
					if(sizex < MAXSIZE)
						sizex ++;
			}
			break;
		case 1:
			if (C)
			{
				if (LEFT)
					if(sizey > 1)
						sizey --;
				if (RIGHT)
					if(sizey < MAXSIZE)
						sizey ++;
			}
			else
			{
				if (R_LEFT)
					if(sizey > 1)
						sizey --;
				if (R_RIGHT)
					if(sizey < MAXSIZE)
						sizey ++;
			}
			break;
		default:;
	}
}

void help()
{
	kbd_Repeat();

	if (R_ESC || R_UP)
	{
		menumode = 1;
		SetLoopFunc(menu);
		return;
	}
}
