// C Header File
// Created 15/02/2004; 14:47:26

#ifndef M7_SPRITES_H
#define M7_SPRITES_H

#include <tigcclib.h>
#include "defines.h"
#include "arch.h"


typedef struct LoadedSprite
{
	unsigned short NbScales;			// max number of sprite scaled sizes (including original size)
	unsigned short NbImg;					// number of images for every animation and every direction including mask images and -if grayscaled- second plane images for only one scaled size
	unsigned short NbAnim;				// number of animations (ex : stopped, walking, shooting -> 3 animations)
	unsigned short NbDir;					// number of directions available for drawing the sprite (usually 8)
	unsigned short * NbImgAnim;		// number of images for the motion of each animation (ex : stopped -> 1, walking -> 2 or 3)
	unsigned short * ScaledSizes;	// [Width (8 pix step), Height for minimum size, ... , W, H for maximum size]
	unsigned char * Data;
	unsigned char * ImgData;
}LoadedSprite;

typedef struct DisplayedSprite
{
	unsigned short Xscreen;
	unsigned short Yscreen;
	unsigned short Zscreen;
	unsigned short width;	// Width (8 pix step) of the current image
	unsigned short height; // Height of the current image
	unsigned char * img; // pointer to the displayed image
											// (if grayscaled, it points to the dark plane. Light plane is width*height bytes after)
	unsigned char * mask; // pointer to the displayed image mask
}DisplayedSprite;

typedef struct Sprite
{
	unsigned short id; // ID of the sprite
	short x; // x on map
	short y; // y on map
	unsigned short z; // altitude of the sprite
	unsigned short dir; // current direction [0,NBDIR[
	unsigned char anim; // current animation [0, NbAnim[
	unsigned char imganim; // current image (in animation)
	unsigned short slot; // sprite data slot number
	unsigned short nbScales; // number of scales for this data
	DisplayedSprite dsprite; // displayed sprite info
}Sprite;

extern unsigned char AddedSprites[MAXSPRT];
extern Sprite * AllSpriteList[MAXSPRT];
extern unsigned char nbDisplayedSprites;
extern DisplayedSprite * Displayed1stSpriteList[MAXSPRT];


void m7sprites_Init();

// Loads the sprite file into ram
// returns the number of sprite data in the table
// returns -1 if no slot is available 
short m7sprites_LoadSprite(ARCH * arch, const char * fileName);

// Frees the slot used by the LoadedSprite in the table
void m7sprites_FreeSprite(unsigned short slot);

// Frees all slots used by LoadedSprites in the table
void m7sprites_FreeAllSprite();

// Adds a new sprite, returns the ID given for this sprite
// return -1 in case of failure
short m7sprites_AddSprite(unsigned short slot);
short m7sprites_Add1stSprite(unsigned short slot);

// Removes the sprite with id ID
void m7sprites_RemoveSprite(unsigned short id);

// Removes all sprites
void m7sprites_RemoveAllSprites();

// Animates the sprite
void m7sprites_Animate(unsigned short id);

// Send DisplayedSprite to DisplayedSpriteList and fills the fields
void m7sprites_SetDisplayed(unsigned short id, 
														unsigned short Xscreen,
														unsigned short Yscreen,
														unsigned short Zscreen,
														unsigned short cameraDir,
														unsigned short dir,
														unsigned short scale);

void m7sprites_Set1stDisplayed(unsigned short id, unsigned short dir);

// Sorts the DisplayedSpriteList in order to have the higher in the screen first
void m7sprites_SortDisplayedSpriteList();

#if ENABLE_GRAYSCALE == 1
void m7sprite_Display(unsigned short num, unsigned char *Plane0, unsigned char *Plane1);
void m7sprite_Display1stPlane(unsigned short id, unsigned char *Plane0, unsigned char *Plane1);
#else
void m7sprite_Display(unsigned short num, unsigned char *Plane0);
void m7sprite_Display1stPlane(unsigned short id, unsigned char *Plane0);
#endif

#endif
